/*
* Copyright (C) 2024, KylinSoft Co., Ltd.
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <https://www.gnu.org/licenses/>.
*
* Authors: zy-yuan1 <zhangyuanyuan1@kylinos.cn>
*
*/

import QtQuick 2.15
import QtQuick.Controls 2.5
//ukui-quick
import org.ukui.quick.items 1.0 as UkuiItem
import org.ukui.quick.platform 1.0 as UkuiPlatform

UkuiItem.DtThemeBackground {
    id: rootItem
    property bool editable: false
    property bool enableLayoutMirroring: false
    property string defaultText: ""
    property string itemDisplayText: defaultText
    property int contentTextWidth: textDisplay.contentWidth
    property bool isDefaultText : false

    signal textEditingFinished()

    implicitWidth: 100
    implicitHeight: 32
    color: "transparent"
    borderColor: editable ? UkuiPlatform.GlobalTheme.kBrandNormal : UkuiPlatform.GlobalTheme.kGrayAlpha0
    border.width : 2
    radius: 4

    MouseArea {
        id: rootMouseArea
        anchors.fill: parent
        hoverEnabled: true
        preventStealing: true
        cursorShape: editable ? Qt.IBeamCursor : Qt.ArrowCursor
        onClicked: {
            if (!editable) {
                // 点击进入编辑状态
                editable = true;
            }
        }
    }

    onEditableChanged: {
        if (editable) {
            // 单击进入textInput可输入状态，需要将焦点强制转移给textInput
            textInput.forceActiveFocus();
            textInput.selectAll();
        }

        // 更新Text显示文本
        updateItemDisplayText();
        textInput.text = itemDisplayText
    }

    TextInput {
        id: textInput
        anchors.fill: parent
        anchors.centerIn : parent
        anchors.leftMargin : 4
        anchors.rightMargin : 4
        horizontalAlignment: TextInput.AlignLeft
        verticalAlignment: TextInput.AlignVCenter
        LayoutMirroring.enabled: enableLayoutMirroring

        clip: true
        maximumLength : 32
        focus: true
        selectByMouse: true
        selectionColor: UkuiPlatform.GlobalTheme.kBrandNormal.pureColor
        visible: editable

        text: itemDisplayText

        onEditingFinished: {
            editable = false;
            textEditingFinished()
        }

        Keys.priority: Keys.AfterItem
        Keys.onPressed: {
            event.accepted = true
        }
    }

    UkuiItem.DtThemeText {
        id: textDisplay
        anchors.fill: parent
        anchors.centerIn: parent
        horizontalAlignment: Text.AlignLeft
        LayoutMirroring.enabled: enableLayoutMirroring
        verticalAlignment: Text.AlignVCenter

        text: itemDisplayText
        elide: enableLayoutMirroring ? Text.ElideLeft : Text.ElideRight
        visible: !editable

        ToolTip.text: textDisplay.text
        ToolTip.delay: 400
        ToolTip.visible: !editable && rootMouseArea.containsMouse && textDisplay.truncated
    }

    function updateItemDisplayText() {
        itemDisplayText = textInput.length > 0 ? textInput.displayText : rootItem.defaultText
        isDefaultText = textInput.length > 0 ? false : true
    }
}
